@@ -294,7 +294,7 @@ void KSaneScanThread::copyToScanData(int readBytes)
 
     qDebug() << "Format" << m_params.format
              << "and depth" << m_params.depth
-             << "is not yet suppoeted by libksane!";
+             << "is not yet supported by libksane!";
     m_readStatus = READ_ERROR;
     return;
 }
