@@ -38,7 +38,7 @@ class KSaneViewer : public QGraphicsView
 {
     Q_OBJECT
 public:
-    explicit KSaneViewer(QImage *img, QWidget *parent = 0);
+    explicit KSaneViewer(QImage *img, QWidget *parent = nullptr);
     ~KSaneViewer();
 
     void setQImage(QImage *img);
@@ -47,7 +47,7 @@ public:
     * \param area this parameter determine the area of the reduced sized image. */
     void findSelections(float area = 10000.0);
 
-    QSize sizeHint() const Q_DECL_OVERRIDE;
+    QSize sizeHint() const override;
 
 public Q_SLOTS:
 
@@ -96,11 +96,11 @@ Q_SIGNALS:
     void newSelection(float tl_x, float tl_y, float br_x, float br_y);
 
 protected:
-    void wheelEvent(QWheelEvent *e) Q_DECL_OVERRIDE;
-    void mousePressEvent(QMouseEvent *e) Q_DECL_OVERRIDE;
-    void mouseReleaseEvent(QMouseEvent *e) Q_DECL_OVERRIDE;
-    void mouseMoveEvent(QMouseEvent *e) Q_DECL_OVERRIDE;
-    void drawBackground(QPainter *painter, const QRectF &rect) Q_DECL_OVERRIDE;
+    void wheelEvent(QWheelEvent *e) override;
+    void mousePressEvent(QMouseEvent *e) override;
+    void mouseReleaseEvent(QMouseEvent *e) override;
+    void mouseMoveEvent(QMouseEvent *e) override;
+    void drawBackground(QPainter *painter, const QRectF &rect) override;
 
 private:
     void updateSelVisibility();
@@ -115,6 +115,8 @@ private:
     int  refineRow(int fromRow, int toRow, int rowStart, int rowEnd);
     int  refineColumn(int fromCol, int toCol, int colStart, int colEnd);
 
+    QPointF scenePos(QMouseEvent *e) const;
+
     struct Private;
     Private *const d;
 
