@@ -460,7 +460,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
     }
 
     // do the connections of the option parameters
-    for (i = 1; i < d->m_optList.size(); ++i) {
+    for (i = 0; i < d->m_optList.size(); ++i) {
         //qDebug() << d->m_optList.at(i)->name();
         connect(d->m_optList.at(i), SIGNAL(optsNeedReload()), d, SLOT(optReload()));
         connect(d->m_optList.at(i), SIGNAL(valsNeedReload()), d, SLOT(scheduleValReload()));
@@ -547,6 +547,16 @@ QImage KSaneWidget::toQImageSilent(const QByteArray &data,
                                    int height,
                                    int bytes_per_line,
                                    ImageFormat format)
+{
+    return KSaneWidget::toQImageSilent(data, width, height, bytes_per_line, currentDPI(), format);
+}
+
+QImage KSaneWidget::toQImageSilent(const QByteArray &data,
+                                   int width,
+                                   int height,
+                                   int bytes_per_line,
+                                   int dpi,
+                                   ImageFormat format)
 {
     QImage img;
     int j = 0;
@@ -619,7 +629,7 @@ QImage KSaneWidget::toQImageSilent(const QByteArray &data,
         qDebug() << "Unsupported conversion";
         break;
     }
-    float dpm = currentDPI() * (1000.0 / 25.4);
+    float dpm = dpi * (1000.0 / 25.4);
     img.setDotsPerMeterX(dpm);
     img.setDotsPerMeterY(dpm);
     return img;
@@ -649,6 +659,16 @@ void KSaneWidget::scanFinal()
     }
 }
 
+void KSaneWidget::startPreviewScan()
+{
+    if (d->m_btnFrame->isEnabled()) {
+        d->startPreviewScan();
+    } else {
+        // if the button frame is disabled, there is no open device to scan from
+        emit scanDone(KSaneWidget::ErrorGeneral, QStringLiteral(""));
+    }
+}
+
 void KSaneWidget::scanCancel()
 {
     if (d->m_scanThread->isRunning()) {
@@ -671,7 +691,7 @@ void KSaneWidget::getOptVals(QMap <QString, QString> &opts)
     opts.clear();
     QString tmp;
 
-    for (int i = 1; i < d->m_optList.size(); i++) {
+    for (int i = 0; i < d->m_optList.size(); i++) {
         option = d->m_optList.at(i);
         if (option->getValue(tmp)) {
             opts[option->name()] = tmp;
@@ -698,14 +718,47 @@ bool KSaneWidget::getOptVal(const QString &optname, QString &value)
 
 int KSaneWidget::setOptVals(const QMap <QString, QString> &opts)
 {
+    if (d->m_scanThread->isRunning() ||
+            d->m_previewThread->isRunning()) {
+        return -1;
+    }
+
+    QMap <QString, QString> optionMapCopy = opts;
+
     QString tmp;
     int i;
     int ret = 0;
 
+    const QString sourceOptionSaneName = QStringLiteral(SANE_NAME_SCAN_SOURCE);
+    const QString modeOptionSaneName   = QStringLiteral(SANE_NAME_SCAN_MODE);
+
+    KSaneOption *opt;
+
+    // Priorize source option
+    if (optionMapCopy.contains(sourceOptionSaneName)) {
+        if ((opt = d->getOption(sourceOptionSaneName)) != nullptr) {
+            if (opt->setValue(optionMapCopy[sourceOptionSaneName])) {
+                ret++;
+            }
+        }
+        optionMapCopy.remove(sourceOptionSaneName);
+    }
+
+    // Priorize mode option
+    if (optionMapCopy.contains(modeOptionSaneName)) {
+        if ((opt = d->getOption(modeOptionSaneName)) != nullptr) {
+            if (opt->setValue(optionMapCopy[modeOptionSaneName])) {
+                ret++;
+            }
+        }
+        optionMapCopy.remove(modeOptionSaneName);
+    }
+
+    // Update remaining options
     for (i = 0; i < d->m_optList.size(); i++) {
-        if (opts.contains(d->m_optList.at(i)->name())) {
-            tmp = opts[d->m_optList.at(i)->name()];
-            if (d->m_optList.at(i)->setValue(tmp) == false) {
+        if (optionMapCopy.contains(d->m_optList.at(i)->name())) {
+            tmp = optionMapCopy[d->m_optList.at(i)->name()];
+            if (d->m_optList.at(i)->setValue(tmp)) {
                 ret++;
             }
         }
@@ -732,8 +785,8 @@ int KSaneWidget::setOptVals(const QMap <QString, QString> &opts)
     }
 
     // special handling for non-sane option
-    if (opts.contains(InvetColorsOption)) {
-        tmp = opts[InvetColorsOption];
+    if (optionMapCopy.contains(InvetColorsOption)) {
+        tmp = optionMapCopy[InvetColorsOption];
         if ((tmp.compare(QStringLiteral("true"), Qt::CaseInsensitive) == 0) ||
                 (tmp.compare(QStringLiteral("1")) == 0)) {
             d->m_invertColors->setChecked(true);
@@ -746,6 +799,11 @@ int KSaneWidget::setOptVals(const QMap <QString, QString> &opts)
 
 bool KSaneWidget::setOptVal(const QString &option, const QString &value)
 {
+    if (d->m_scanThread->isRunning() ||
+            d->m_previewThread->isRunning()) {
+        return false;
+    }
+
     KSaneOption *opt;
 
     if ((opt = d->getOption(option)) != nullptr) {
