@@ -34,6 +34,7 @@
 
 #include <QDebug>
 #include <QIcon>
+#include <QLocale>
 
 namespace KSaneIface
 {
@@ -328,9 +329,6 @@ bool KSaneOptCombo::setValue(const QString &val)
     if (state() == STATE_HIDDEN) {
         return false;
     }
-    if (val == m_currentText) {
-        return true;
-    }
 
     unsigned char data[4];
     void* data_ptr = nullptr;
@@ -343,8 +341,8 @@ bool KSaneOptCombo::setValue(const QString &val)
     switch (m_optDesc->type) {
     case SANE_TYPE_INT:
         tmp = val.left(val.indexOf(QLatin1Char(' '))); // strip the unit
-        // accept float formating of the string
-        i = (int)(tmp.toFloat(&ok));
+        // accept float formatting of the string
+        i = (int)(QLocale::system().toFloat(tmp,&ok));
         if (ok == false) {
             return false;
         }
@@ -353,7 +351,7 @@ bool KSaneOptCombo::setValue(const QString &val)
         break;
     case SANE_TYPE_FIXED:
         tmp = val.left(val.indexOf(QLatin1Char(' '))); // strip the unit
-        f = tmp.toFloat(&ok);
+        f = QLocale::system().toFloat(tmp,&ok);
         if (ok == false) {
             return false;
         }
